pro syncdop,overs=overs,size=size
; synchronize dop.out with input data by recompiling/rerunning.
; The basic resolution in the dopmap is determined by formula 'A' below: if phase 
; resolution gives smaller velocity bin midway between center and edge of dopmap, 
; take this, otherwise take the velocity bin size of the input spectrum. 
; size: if set, compute only size required for proj matrices, then return
; look at emap.par and if needed make it consistent with input data.
; overs: resolution factor in dopmap compared with no of points in wavelength
;        of input data. overs=1 (default) gives same resolution as data
; size: if set, only compute and print memory requirements
get_lun,lun
openr,lun,'dopin'
np=1
nvp=1
readf,lun,np,nvp                ; read size parameters from 'dopin'
close,lun

openr,lun,'emap.par'
tem=strarr(8)
tt=''
readf,lun,tt
tem(0)=tt
close,lun
tem(1)='      parameter (nd=npm*nvpm,nr=0.8*nvm*nvm)'
tem(2)='      parameter (nt=nvpm*npm+nvm*nvpm*3+2*npm*nvm)'
tem(3)='      parameter (nri=0.9*nvm*nt/nd,ndi=0.9*nvm*nt/nr)'
tem(4)='c parameters for emap routines'
tem(5)='      parameter (nf=nd,nfp=ndi,nrp=nri)'
tem(6)='      parameter (ni=nvm,nj=nvm)'
tem(7)='      parameter (nch=1,nsu=1)'
nli=8

l1=strcompress(tem(0),/remove_all)
l2=str_sep(l1,'=')
l3=str_sep(l2(1),',')
reads,l3,npm
l3=str_sep(l2(2),',')
reads,l3,nvpm
reads,l2(3),nvm
nv0=fix(overs*nvp)
nv=max([nv0,fix(np/3.<1.5*nv0)])    ; formula A for adequate resolution in dopspace.
nv=(nv/2)*2
if ((nv ne nvm) or (np ne npm) or (nvp ne nvpm)) then begin
; emap.par needs to be renewed
  tem(0)='      parameter (npm='+ $
    string(format='(i4,'',nvpm='',i4,'',nvm='',i4)',np,nvp,nv)+')'
  openw,lun,'emap.par'
  for i=0,nli-1 do printf,lun,tem(i)
  close,lun
endif

nd=npm*nvpm
nr=0.8*nvm*nvm
nt=nvpm*npm+nvm*nvpm*3+2*npm*nvm
prmsize=0.9*nvm*nt+0.9*nvm*nt
if keyword_set(size) then begin
  print,'sum proj matrices',8*prmsize,' bytes'
  return
endif
spawn,'make dop.out'
; extract last line of log file for error diagnosis
spawn,'tail dop.log',dump,count=nline      
pars=str_sep(strtrim(strcompress(dump(nline-2)),2),' ')
npa=n_elements(pars)
reads,pars(npa-2),rr
reads,pars(npa-1),clim
if rr gt clim then print,'specified reduced chi^2 not reached',rr,clim
free_lun,lun
return
end









